@@ -38,18 +38,27 @@ import javax.swing.AbstractAction;
 import javax.swing.JMenuItem;
 import javax.swing.JPopupMenu;
 import org.biolegato.core.data.sequence.Sequence;
-import org.biolegato.core.main.ProgramWindow;
+import org.biolegato.core.main.BLMain;
 
 /**
+ * A general canvas with more functionality support than JTextArea.
  *
- * @author alvare
+ * This canvas was originally created to support the rectangular selection model,
+ * and has since supported different types of selections, colour schemes, and many
+ * other features.  Please add generic feature support as necessary.
+ *
+ * Also if you wish to add any functionality or handling specific to a plugin,
+ * please feel free to subclass this textarea.
+ *
+ * @author Graham Alvare
+ * @author Brian Fristensky
  */
 public class BLTextArea extends JComponent implements FocusListener, SelectionListener, Editable, UndoRedoable,SeqDocListener, KeyListener, MouseMotionListener, MouseListener {
 
     /**
      * The biolegato instance to use to obtain data from
      */
-    protected ProgramWindow instance = null;
+    protected BLMain instance = null;
     /**
      * If true new text will overwrite existing text.
      */
@@ -97,7 +106,7 @@ public class BLTextArea extends JComponent implements FocusListener, SelectionLi
      */
     protected ColourMap greyedOutMap = new MonochromeColourMap(Color.BLACK, Color.LIGHT_GRAY, Color.DARK_GRAY, Color.WHITE);
     /**
-     * Handles printing text
+     * The colour map to use when the textarea is not greyed out.
      */
     protected ColourMap normalMap = new MonochromeColourMap(Color.BLACK, Color.WHITE, Color.WHITE, Color.BLUE);
     /**
@@ -133,9 +142,8 @@ public class BLTextArea extends JComponent implements FocusListener, SelectionLi
      * Creates a new instance of BLTextArea
      *
      * @param instance the BioLegat instance to attach the textarea to
-     * @param data the document to use for the textarea
      */
-    public BLTextArea (ProgramWindow instance) {
+    public BLTextArea (BLMain instance) {
         // initialize variables
         this.instance = instance;
 
@@ -177,7 +185,7 @@ public class BLTextArea extends JComponent implements FocusListener, SelectionLi
         }));
 
         // set display
-        setFont(ProgramWindow.DEFAULT_FONT);
+        setFont(BLMain.DEFAULT_FONT);
         setForeground(Color.BLACK);
         setBackground(Color.WHITE);
         setFocusTraversalKeysEnabled(false);
@@ -252,13 +260,13 @@ public class BLTextArea extends JComponent implements FocusListener, SelectionLi
 //*******************//
 ///////////////////////
     /**
-     * Changes the colour map for unselected text.
+     * Changes the current colour map for unselected text.
      *
      * @param c the colour map to use
      */
     public void setColourMap (ColourMap c) {
         colourMap = c;
-	normalMap = colourMap;
+        normalMap = colourMap;
         c.setBackground(getBackground());
         repaint();
     }
@@ -266,12 +274,12 @@ public class BLTextArea extends JComponent implements FocusListener, SelectionLi
     /**
      * Changes the background of the canvas.
      *
-     * @param c the new colour for the canvas background.
+     * @param c the new colour for the normal canvas background.
      */
     @Override
     public void setBackground (Color c) {
         super.setBackground(c);
-        colourMap.setBackground(c);
+        normalMap.setBackground(c);
     }
 
     /**
@@ -285,7 +293,7 @@ public class BLTextArea extends JComponent implements FocusListener, SelectionLi
     }
 
     /**
-     * Paints the current textarea
+     * Paints the current textarea (this calls paint with the gfx's clip bounds)
      *
      * @param gfx the graphics instance to paint the window to.
      */
@@ -489,7 +497,7 @@ public class BLTextArea extends JComponent implements FocusListener, SelectionLi
                             }
                             break;
                         default:
-                            if (instance.getProperty("debug").equalsIgnoreCase("true")) {
+                            if ("true".equalsIgnoreCase(instance.getProperty("debug"))) {
                                 org.biolegato.core.main.BLMain.warning("Unhandled key pressed --- getKeyChar() = " + ((int) event.getKeyChar()) + "\tgetKeyCode() = " + event.getKeyCode(), "BLTextArea");
                             }
                             break;
@@ -1004,7 +1012,7 @@ public class BLTextArea extends JComponent implements FocusListener, SelectionLi
         colourMap = normalMap;
         setForeground(Color.BLACK);
         setBackground(Color.WHITE);
-	repaint();
+        repaint();
     }
 
     /**
