@@ -8,9 +8,16 @@
  */
 package org.biolegato.core.data.sequence;
 
+import java.awt.datatransfer.Clipboard;
+import java.awt.datatransfer.ClipboardOwner;
+import java.awt.datatransfer.DataFlavor;
+import java.awt.datatransfer.Transferable;
+import java.awt.datatransfer.UnsupportedFlavorException;
+import java.io.IOException;
 import java.util.Hashtable;
 import java.util.LinkedList;
 import java.util.Map;
+import org.biolegato.core.data.sequence.SequenceListener;
 
 /**
  * This class is used to contain all sequence related functions and data.
@@ -42,13 +49,8 @@ import java.util.Map;
  * @author Graham Alvare
  * @author Brian Fristensky
  */
-public class Sequence extends Hashtable<String,Object> implements Cloneable {
+public class Sequence extends Hashtable<String, Object> implements Cloneable {
 
-    /**
-     * This linked list stores all sequence listeners
-     */
-    private final LinkedList<SequenceListener> listeners = new LinkedList<SequenceListener>();
-    
     /**
      * This enum is used for typing/storing sequence types.
      * This method of storage ensures proper type casting.
@@ -265,6 +267,48 @@ public class Sequence extends Hashtable<String,Object> implements Cloneable {
 
     }
 
+    /**
+     * Used for typing characters within a sequence.
+     * This is especially useful for intrinsic functions which rely upon detecting alignment gaps,
+     * ambiguous characters and unambiguous characters (such as certain colour maps).
+     */
+    public static enum CharacterClass {
+
+        /**
+         * Alignment gap class.
+         */
+        ALIGNMENT,
+        /**
+         * Ambiguous character class.
+         */
+        AMBIGUOUS,
+        /**
+         * Unambiguous character class.
+         */
+        UNAMBIGUOUS;
+
+    }
+
+    /**
+     * This linked list stores all sequence listeners
+     */
+    private final LinkedList<SequenceListener> listeners = new LinkedList<SequenceListener>();
+    
+    /**
+     * Keeps track of the maximum group number in usage.
+     */
+    private static int maxGroup = 1;
+    
+    /**
+     * A data flavour representing the raw Sequence data type.
+     */
+    public static final DataFlavor sequenceFlavour = new DataFlavor(Sequence.class, "Raw BioLegato sequence");
+    
+    /**
+     * A hashtable used to map all groups to sequences 
+     */
+    private final static Hashtable<Integer, LinkedList<Sequence>> group2seq = new Hashtable<Integer, LinkedList<Sequence>>();
+    
     /**
      * Stores the default values for each intrinsic Sequence data key.
      */
@@ -275,19 +319,24 @@ public class Sequence extends Hashtable<String,Object> implements Cloneable {
 
         private static final long serialVersionUID = 7526472295622777006L;
         {
-            put("modified", Boolean.FALSE);             // stores that by default the sequence has not been modified
-            put("name", "New Sequence");                //
-            put("direction", Direction.FROM5TO3);
-            put("strandedness", Strandedness.SINGLE);
-            put("topology", Topology.LINEAR);
-            put("type", Type.DNA);
-            put("protect_align", Boolean.FALSE);
-            put("protect_ambig", Boolean.TRUE);
-            put("protect_unambig", Boolean.TRUE);
+            super.put("modified", Boolean.FALSE);             // stores that by default the sequence has not been modified
+            super.put("name", "New Sequence");                //
+            super.put("direction", Direction.FROM5TO3);
+            super.put("strandedness", Strandedness.SINGLE);
+            super.put("topology", Topology.LINEAR);
+            super.put("type", Type.DNA);
+            super.put("protect_align", Boolean.FALSE);
+            super.put("protect_ambig", Boolean.TRUE);
+            super.put("protect_unambig", Boolean.TRUE);
         }
-	
+
 
     };
+
+    /**
+     * Used for serialization purposes.
+     */
+    private static final long serialVersionUID = 7526472295622777024L;
     
     /**
      * Constructs new instances of Sequence objects.
@@ -302,8 +351,11 @@ public class Sequence extends Hashtable<String,Object> implements Cloneable {
      * @param string the string to use for the "sequence" field in the sequence data.
      */
     public Sequence (String string) {
-        this(defaults);
-        put("sequence", string);
+	if (string != null) {
+	    put("sequence", string);
+	}
+        detectType();
+	super.put("modified", Boolean.FALSE);
     }
 
     /**
@@ -312,7 +364,12 @@ public class Sequence extends Hashtable<String,Object> implements Cloneable {
      * @param data the map of values to use for the sequence.
      */
     public Sequence (Map<String, Object> data) {
-        super(data);
+	this((data.containsKey("sequence") ? data.get("sequence").toString() : ""));
+	
+	for (String key : data.keySet()) {
+	    put(key, data.get(key));
+	}
+	super.put("modified", Boolean.FALSE);
     }
     
 //////////////////////////
@@ -347,11 +404,12 @@ public class Sequence extends Hashtable<String,Object> implements Cloneable {
      * @param key the field name for the sequence data
      * @return the value of the field
      */
-    public Object getField (String key) {
+    public Object get (Object key) {
         // ensure that the sequence contains the desired field
         // if the field is not contained in the hashtable parent,
         // return the default field value instead
-        return (containsKey(key) ? get(key) : getDefault(key));
+	String skey = key.toString().toLowerCase();
+        return (containsKey(skey) ? super.get(skey) : getDefault(skey));
     }
 
     /**
@@ -362,7 +420,7 @@ public class Sequence extends Hashtable<String,Object> implements Cloneable {
      * @param key the name of the field in the sequence.
      * @param value the new value of the field.
      */
-    public void setField (String key, Object value) {
+    public Object put (String key, Object value) {
         // ensure the key is not null
         if (key == null) {
             throw new NullPointerException("Cannot call Sequence.setField(" + key + "," + value + ")");
@@ -371,14 +429,43 @@ public class Sequence extends Hashtable<String,Object> implements Cloneable {
             if (value == null) {
                 value = getDefault(key);
             }
-            put("modified", Boolean.TRUE);
-            put(key, value);
+	    key = key.toLowerCase();
+            super.put("modified", Boolean.TRUE);
+            super.put(key, value);
         }
         for (SequenceListener listener : listeners) {
             listener.sequenceChanged(this, key);
         }
+	return value;
     }
 
+    /**
+     * Inserts a string into field within the sequence.
+     *
+     * @param field the field to alter.
+     * @param column the column number to do the insertion at.
+     * @param string the text to insert.
+     * @return true if the operation was successful, otherwise false.
+     */
+    public boolean insertField (final String field, final int column, final String string) {
+	String current = get(field).toString();
+
+	// figure out how to insert the new data
+	// 1. if the offset is in the middle of the sequence
+	// 2. if the offset is at the end of the sequence
+	// 3. if the offset is at the beginning of the sequence
+	if (column > 0 && column < current.length()) {
+	    put(field, current.substring(0, column) + string + current.substring(column));
+	} else if (column >= current.length()) {
+	    put(field, current + string);
+	} else {
+	    put(field, string + current);
+	}
+	
+	// always return true
+	return true;
+    }
+    
     /**
      * This function obtains the default value of a field for a Sequence.
      * This is done by accessing the defaults hashtable.
@@ -390,6 +477,7 @@ public class Sequence extends Hashtable<String,Object> implements Cloneable {
         // ensure that the sequence contains the desired field
         // if the field is not contained in the hashtable parent,
         // return a blank string instead
+	key = key.toLowerCase();
         return (defaults.containsKey(key) ? defaults.get(key) : "");
     }
 
@@ -406,7 +494,7 @@ public class Sequence extends Hashtable<String,Object> implements Cloneable {
         // itterate through each field in the sequence and print it
         // NOTE: append is faster than + or concat operators
         for (String key : keySet()) {
-            result.append("\t" + key + "=" + getField(key) +"\n");
+            result.append("\t" + key + "=" + get(key) +"\n");
         }
         return result.toString();
     }
@@ -427,10 +515,10 @@ public class Sequence extends Hashtable<String,Object> implements Cloneable {
 
         // ensure that the split occurs at a valid location,
         // otherwise make the sequence to return a blank sequence
-        if (start <= split.getField("sequence").toString().length()) {
-            split.setField("sequence", split.getField("sequence").toString().substring(start));
+        if (start <= split.get("sequence").toString().length()) {
+            split.put("sequence", split.get("sequence").toString().substring(start));
         } else {
-            split.setField("sequence", "");
+            split.put("sequence", "");
         }
 
         return split;
@@ -453,8 +541,8 @@ public class Sequence extends Hashtable<String,Object> implements Cloneable {
         // ensure that the length parameter is less than the length of the sequence;
         // also ensure that the length parameter is greater than or equal to zero
         // if neither are ensured don't truncate length off of the end of the sequence
-        if (end >= 0 && end <= split.getField("sequence").toString().length()) {
-            split.setField("sequence", split.getField("sequence").toString().substring(0, end));
+        if (length >= 0 && length <= split.get("sequence").toString().length()) {
+            split.put("sequence", split.get("sequence").toString().substring(0, length));
         }
         return split;
     }
@@ -469,4 +557,108 @@ public class Sequence extends Hashtable<String,Object> implements Cloneable {
         // since Sequence implements Map as a child of Hashtable, this code works
         return new Sequence(this);
     }
+
+    /**
+     * This function detects the type of a sequence and sets the type field for that sequence appropriately.
+     *
+     * @param seq the sequence to set the type for.
+     */
+    private void detectType() {
+	String data;
+	Sequence.Type result = Sequence.Type.DNA;
+
+	if (containsKey("sequence")) {
+	    data = get("sequence").toString().toUpperCase().trim();
+	    if (data.indexOf('U') >= 0) {
+		result = Sequence.Type.RNA;
+	    } else if (data.indexOf('F') >= 0 || data.indexOf('E') >= 0 || data.indexOf('J') >= 0
+		    || data.indexOf('L') >= 0 || data.indexOf('O') >= 0 || data.indexOf('Q') >= 0
+		    || data.indexOf('X') >= 0 || data.indexOf('Z') >= 0) {
+		result = Sequence.Type.PROTEIN;
+	    }
+	    put("type", result);
+	}
+    }
+
+    public CharacterClass charType (char test) {
+        CharacterClass type = CharacterClass.ALIGNMENT;
+
+        if (Type.DNA.equals(get("type")) || Type.RNA.equals(get("type"))) {
+            switch (test) {
+                case 'R': case 'Y': case 'W': case 'S': case 'M': case 'K': case 'H':
+                case 'B': case 'V': case 'I': case 'D': case 'N': case 'r': case 'y':
+                case 'w': case 's': case 'm': case 'k': case 'h': case 'b': case 'v':
+                case 'i': case 'd': case 'n':
+                    type = CharacterClass.AMBIGUOUS;
+                    break;
+                case 'A': case 'C': case 'G': case 'T': case 'U':
+                case 'a': case 'c': case 'g': case 't': case 'u':
+                    type = CharacterClass.UNAMBIGUOUS;
+                    break;
+                default:
+                    type = CharacterClass.ALIGNMENT;
+                    break;
+            }
+        } else if (Type.PROTEIN.equals(get("type"))) {
+            switch (test) {
+                case 'B': case 'Z': case 'X': case '*': case 'b': case 'z': case 'x':
+                    type = CharacterClass.AMBIGUOUS;
+                    break;
+                case 'F': case 'M': case 'P': case 'Y': case 'N': case 'E': case 'R':
+                case 'J': case 'L': case 'V': case 'T': case 'H': case 'K': case 'C':
+                case 'G': case 'O': case 'I': case 'S': case 'A': case 'Q': case 'D':
+                case 'W': case 'f': case 'm': case 'p': case 'y': case 'n': case 'e':
+                case 'r': case 'j': case 'l': case 'v': case 't': case 'h': case 'k':
+                case 'c': case 'g': case 'o': case 'i': case 's': case 'a': case 'q':
+                case 'd': case 'w':
+                    type = CharacterClass.UNAMBIGUOUS;
+                    break;
+                default:
+                    type = CharacterClass.ALIGNMENT;
+                    break;
+            }
+        }
+        return type;
+    }
+
+    public static void group (Sequence[] sequences) {
+	LinkedList<Sequence> add = new LinkedList<Sequence>();
+	for (Sequence seq : sequences) {
+	    ungroup(new Sequence[]{seq});
+	    seq.put("group", new Integer(maxGroup));
+	    add.add(seq);
+	}
+	group2seq.put(new Integer(maxGroup), add);
+	maxGroup++;
+    }
+
+    public static void ungroup (Sequence[] sequences) {
+	Integer groupNumber;
+	LinkedList<Sequence> group;
+	for (Sequence seq : sequences) {
+	    if (seq.get("group") instanceof Integer) {
+		groupNumber = (Integer) seq.get("group");
+		group = getgroup(groupNumber);
+		group.remove(seq);
+		if (group.isEmpty()) {
+		    while (maxGroup > 1 && maxGroup < group2seq.size() && group2seq.get(maxGroup - 1).isEmpty()) {
+			maxGroup--;
+		    }
+		    group2seq.remove(groupNumber);
+		}
+		seq.put("group", null);
+	    }
+	}
+    }
+    
+    public static LinkedList<Sequence> getgroup(Integer groupNumber) {
+	LinkedList<Sequence> group = null;
+	
+	if (groupNumber != null && group2seq.containsKey(groupNumber)) {
+	    group = group2seq.get(groupNumber);
+	} else {
+	    group = new LinkedList<Sequence>();
+	}
+	return group;
+    }
 }
