@@ -9,7 +9,7 @@
 
 package org.biolegato.core.plugintypes;
 
-import org.biolegato.core.main.ProgramWindow;
+import org.biolegato.core.main.BLMain;
 
 /**
  * An interface for menu reader objects.  All subclasses are expected to parse menu objects into BioLegato.
@@ -20,8 +20,6 @@ import org.biolegato.core.main.ProgramWindow;
 public abstract class MenuType {
     /**
      * Reads a menu into the given window/menu instances.
-     *
-     * @param program the BioLegato program instance to load the menu
      */
-    public static void loadMenu (final ProgramWindow program) {}
+    public static void loadMenu () {}
 }
\ No newline at end of file
